@@ -1,5 +1,5 @@
 /* GMM (C) 2018 Stephane Charette <stephanecharette@gmail.com>
- * $Id: TabCanvas.cpp 2688 2018-12-07 21:20:05Z stephane $
+ * $Id: TabCanvas.cpp 2694 2018-12-09 21:52:31Z stephane $
  */
 
 #include "GMM.hpp"
@@ -403,7 +403,7 @@ Image TabCanvas::draw_chart(const size_t width, const size_t height)
 	const float width_float				= static_cast<float>(width);
 	const float height_float			= static_cast<float>(height);
 	const float h_step					= 1.0f / static_cast<float>(number_of_buckets);
-	const float v_step					= 1.0f / 5.0f; // 20%, 40%, 60%, 80%, and 100%
+	const float v_step					= 1.0f / 10.0f; // 10%, 20%, 30%, etc...
 	const float width_of_columns		= h_step * width_float;
 	const float one_seventh_of_column	= 1.0f * width_of_columns / 7.0f;
 	const float three_seventh_of_column	= 3.0f * width_of_columns / 7.0f;
@@ -481,16 +481,34 @@ Image TabCanvas::draw_chart(const size_t width, const size_t height)
 	// add % text labels at the bottom of the image
 	g.setColour(Colours::black);
 	const float label_h = 15;
-	const float label_y = height - label_h;
-	int percentage	= percentage_of_first_bucket;
-	for (float x = 0.0; x < 1.0; x += h_step)
+	if (true)
 	{
-		const float label_x = width_float * x		+ edge_offset;
-		const float label_w = width_float * h_step	- edge_offset * 2;
-		g.drawText(String(percentage)+"%", label_x, label_y, label_w, label_h, Justification::centredBottom);
-		percentage ++;
+		const float label_y = height - label_h;
+		int percentage	= percentage_of_first_bucket;
+		for (float x = 0.0; x < 1.0; x += h_step)
+		{
+			const float label_x = width_float * x		+ edge_offset;
+			const float label_w = width_float * h_step	- edge_offset * 2;
+			g.drawText(String(percentage)+"%", label_x, label_y, label_w, label_h, Justification::centredBottom);
+			percentage ++;
+		}
 	}
 
+	#if 0
+	// add % text labels on the right-hand-side of the image
+	g.setColour(Colours::lightgrey);
+	if (true)
+	{
+		const float label_w = 50.0;
+		const float label_x = width_float - label_w - edge_offset;
+		for (float y = v_step; y < 1.0; y += v_step)
+		{
+			const float label_y = y * height_float - label_h + edge_offset/2;
+			g.drawText(String(std::round(100.0 * (1.0 - y))) + "%", label_x, label_y, label_w, label_h, Justification::topRight);
+		}
+	}
+	#endif
+
 	// combine the two vectors of errors/warnings, and add them to the top-left of the image
 	auto tmp = gmm().wnd->warnings_and_errors;
 	tmp.insert(tmp.end(), warnings_and_errors.begin(), warnings_and_errors.end());
